'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = undefined;

var _index = require('../util/index.js');

var _Event2 = require('../Event');

var _Event3 = _interopRequireDefault(_Event2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint-disable */


var PointerEvent = function (_Event) {
    _inherits(PointerEvent, _Event);

    function PointerEvent(type) {
        _classCallCheck(this, PointerEvent);

        var _this = _possibleConstructorReturn(this, (PointerEvent.__proto__ || Object.getPrototypeOf(PointerEvent)).call(this, type));

        _this.target = window.canvas;
        _this.currentTarget = window.canvas;
        return _this;
    }

    return PointerEvent;
}(_Event3.default);

exports.default = PointerEvent;


var CLONE_PROPS = [

// MouseEvent
'bubbles', 'cancelable', 'view', 'detail', 'screenX', 'screenY', 'clientX', 'clientY', 'ctrlKey', 'altKey', 'shiftKey', 'metaKey', 'button', 'relatedTarget',

// PointerEvent
'pointerId', 'width', 'height', 'pressure', 'tiltX', 'tiltY', 'pointerType', 'hwTimestamp', 'isPrimary',

// event instance
'pageX', 'pageY', 'timeStamp'];

var CLONE_DEFAULTS = [

// MouseEvent
false, false, null, null, 0, 0, 0, 0, false, false, false, false, 0, null,

// DOM Level 3
0,

// PointerEvent
0, 0, 0, 0, 0, 0, '', 0, false,

// event instance
0, 0, 0];

var POINTER_TYPE = 'touch';

function touchToPointer(type, touch, rawEvent) {
    var e = new PointerEvent(type);

    for (var i = 0; i < CLONE_PROPS.length; i++) {
        var p = CLONE_PROPS[i];
        e[p] = touch[p] || CLONE_DEFAULTS[i];
    }

    e.type = type;
    e.target = window.canvas;
    e.currentTarget = window.canvas;
    e.buttons = typeToButtons(type);
    e.which = e.buttons;

    e.pointerId = (touch.identifier || 0) + 2;
    e.bubbles = true;
    e.cancelable = true;
    // e.detail = this.clickCount;
    e.button = 0;

    e.width = (touch.radiusX || 0.5) * 2;
    e.height = (touch.radiusY || 0.5) * 2;
    e.pressure = touch.force || 0.5;
    e.isPrimary = isPrimaryPointer(touch);
    e.pointerType = POINTER_TYPE;

    // forward modifier keys
    e.altKey = rawEvent.altKey;
    e.ctrlKey = rawEvent.ctrlKey;
    e.metaKey = rawEvent.metaKey;
    e.shiftKey = rawEvent.shiftKey;

    if (rawEvent.preventDefault) {
        e.preventDefault = function () {
            rawEvent.preventDefault();
        };
    }

    return e;
}

function typeToButtons(type) {
    var ret = 0;
    if (type === 'touchstart' || type === 'touchmove' || type === 'pointerdown' || type === 'pointermove') {
        ret = 1;
    }
    return ret;
}

var firstPointer = null;

function isPrimaryPointer(touch) {
    return firstPointer === touch.identifier;
}

function setPrimaryPointer(touch) {
    if (firstPointer === null) {
        firstPointer = touch.identifier;
    }
}

function removePrimaryPointer(touch) {
    if (firstPointer === touch.identifier) {
        firstPointer = null;
    }
}

function eventHandlerFactory(type) {
    return function (rawEvent) {

        var changedTouches = rawEvent.changedTouches;

        for (var i = 0; i < changedTouches.length; i++) {
            var touch = changedTouches[i];

            if (i === 0 && type === 'pointerdown') {
                setPrimaryPointer(touch);
            } else if (type === 'pointerup' || type === 'pointercancel') {
                removePrimaryPointer(touch);
            }

            var event = touchToPointer(type, touch, rawEvent);
            document.dispatchEvent(event);
        }
    };
}

if (swan.onTouchStart) {
    swan.onTouchStart(eventHandlerFactory('pointerdown'));
    swan.onTouchMove(eventHandlerFactory('pointermove'));
    swan.onTouchEnd(eventHandlerFactory('pointerup'));
    swan.onTouchCancel(eventHandlerFactory('pointercancel'));
}
module.exports = exports['default'];