'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _window = require('./window');

var window = _interopRequireWildcard(_window);

var _Event = require('./Event');

var _Event2 = _interopRequireDefault(_Event);

var _HTMLElement = require('./HTMLElement');

var _HTMLElement2 = _interopRequireDefault(_HTMLElement);

var _HTMLVideoElement = require('./HTMLVideoElement');

var _HTMLVideoElement2 = _interopRequireDefault(_HTMLVideoElement);

var _Image = require('./Image');

var _Image2 = _interopRequireDefault(_Image);

var _Audio = require('./Audio');

var _Audio2 = _interopRequireDefault(_Audio);

var _Canvas = require('./Canvas');

var _Canvas2 = _interopRequireDefault(_Canvas);

var _DocumentElement = require('./DocumentElement');

var _DocumentElement2 = _interopRequireDefault(_DocumentElement);

var _Body = require('./Body');

var _Body2 = _interopRequireDefault(_Body);

require('./EventIniter/index.js');

var _location = require('./location');

var _location2 = _interopRequireDefault(_location);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var events = {}; /* eslint-disable */


var document = {
    readyState: 'complete',
    visibilityState: 'visible', // 'visible' , 'hidden'
    hidden: false,
    fullscreen: true,

    URL: _location2.default.href,

    location: window.location,

    scripts: [],
    style: {},

    ontouchstart: null,
    ontouchmove: null,
    ontouchend: null,
    onvisibilitychange: null,

    parentNode: null,
    parentElement: null,

    createElement: function createElement(tagName) {
        tagName = tagName.toLowerCase();
        if (tagName === 'canvas') {
            return new _Canvas2.default();
        } else if (tagName === 'audio') {
            return new _Audio2.default();
        } else if (tagName === 'img') {
            return new _Image2.default();
        } else if (tagName === 'video') {
            return new _HTMLVideoElement2.default();
        }

        return new _HTMLElement2.default(tagName);
    },
    createElementNS: function createElementNS(nameSpace, tagName) {
        return this.createElement(tagName);
    },
    createTextNode: function createTextNode(text) {
        // TODO: Do we need the TextNode Class ???
        return text;
    },
    getElementById: function getElementById(id) {
        if (id === window.canvas.id) {
            return window.canvas;
        }
        return null;
    },
    getElementsByTagName: function getElementsByTagName(tagName) {
        tagName = tagName.toLowerCase();
        if (tagName === 'head') {
            return [document.head];
        } else if (tagName === 'body') {
            return [document.body];
        } else if (tagName === 'canvas') {
            return [window.canvas];
        }
        return [];
    },
    getElementsByTagNameNS: function getElementsByTagNameNS(nameSpace, tagName) {
        return this.getElementsByTagName(tagName);
    },
    getElementsByName: function getElementsByName(tagName) {
        if (tagName === 'head') {
            return [document.head];
        } else if (tagName === 'body') {
            return [document.body];
        } else if (tagName === 'canvas') {
            return [window.canvas];
        }
        return [];
    },
    querySelector: function querySelector(query) {
        if (query === 'head') {
            return document.head;
        } else if (query === 'body') {
            return document.body;
        } else if (query === 'canvas') {
            return window.canvas;
        } else if (query === '#' + window.canvas.id) {
            return window.canvas;
        }
        return null;
    },
    querySelectorAll: function querySelectorAll(query) {
        if (query === 'head') {
            return [document.head];
        } else if (query === 'body') {
            return [document.body];
        } else if (query === 'canvas') {
            return [window.canvas];
        }
        return [];
    },
    addEventListener: function addEventListener(type, listener) {
        if (!events[type]) {
            events[type] = [];
        }
        events[type].push(listener);
    },
    removeEventListener: function removeEventListener(type, listener) {
        var listeners = events[type];

        if (listeners && listeners.length > 0) {
            for (var i = listeners.length; i--; i > 0) {
                if (listeners[i] === listener) {
                    listeners.splice(i, 1);
                    break;
                }
            }
        }
    },
    dispatchEvent: function dispatchEvent(event) {
        var type = event.type;
        var listeners = events[type];

        if (listeners) {
            for (var i = 0; i < listeners.length; i++) {
                listeners[i](event);
            }
        }

        if (event.target && typeof event.target['on' + type] === 'function') {
            event.target['on' + type](event);
        }
    }
};

document.documentElement = new _DocumentElement2.default();
document.head = new _HTMLElement2.default('head');
document.body = new _Body2.default();

function onVisibilityChange(visible) {

    return function () {

        document.visibilityState = visible ? 'visible' : 'hidden';

        var hidden = !visible;
        if (document.hidden === hidden) {
            return;
        }
        document.hidden = hidden;

        var event = new _Event2.default('visibilitychange');

        event.target = document;
        event.timeStamp = Date.now();

        document.dispatchEvent(event);
    };
}

if (swan.onHide) {
    swan.onHide(onVisibilityChange(false));
    swan.onShow(onVisibilityChange(true));
}

exports.default = document;
module.exports = exports['default'];