'use strict';

var sys = cc.sys;
var mgr = _cc && _cc.inputManager;

if (mgr) {
    Object.assign(mgr, {
        getHTMLElementPosition: function getHTMLElementPosition(element) {
            return {
                left: 0,
                top: 0,
                width: window.innerWidth,
                height: window.innerHeight
            };
        },
        getPointByEvent: function getPointByEvent(event, pos) {
            if (event.pageX != null) //not avalable in <= IE8
                return { x: event.pageX, y: event.pageY };

            pos.left = 0;
            pos.top = 0;

            return { x: event.clientX, y: event.clientY };
        },
        registerSystemEvent: function registerSystemEvent(element) {
            if (this._isRegisterEvent) return;

            this._glView = cc.view;
            var selfPointer = this;

            var prohibition = sys.isMobile;
            var supportMouse = 'mouse' in sys.capabilities;
            var supportTouches = 'touches' in sys.capabilities;

            // adapt in BAIDU_GAME
            prohibition = false;
            supportTouches = true;
            supportMouse = false;

            if (supportMouse) {
                //HACK
                //  - At the same time to trigger the ontouch event and onmouse event
                //  - The function will execute 2 times
                //The known browser:
                //  liebiao
                //  miui
                //  WECHAT
                if (!prohibition) {
                    window.addEventListener('mousedown', function () {
                        selfPointer._mousePressed = true;
                    }, false);

                    window.addEventListener('mouseup', function (event) {
                        if (!selfPointer._mousePressed) return;

                        selfPointer._mousePressed = false;

                        var pos = selfPointer.getHTMLElementPosition(element);
                        var location = selfPointer.getPointByEvent(event, pos);
                        if (!cc.rect(pos.left, pos.top, pos.width, pos.height).contains(location)) {
                            selfPointer.handleTouchesEnd([selfPointer.getTouchByXY(location.x, location.y, pos)]);

                            var mouseEvent = selfPointer.getMouseEvent(location, pos, cc.Event.EventMouse.UP);
                            mouseEvent.setButton(event.button);
                            eventManager.dispatchEvent(mouseEvent);
                        }
                    }, false);
                }

                // register canvas mouse event
                var EventMouse = cc.Event.EventMouse;
                var _mouseEventsOnElement = [!prohibition && ["mousedown", EventMouse.DOWN, function (event, mouseEvent, location, pos) {
                    selfPointer._mousePressed = true;
                    selfPointer.handleTouchesBegin([selfPointer.getTouchByXY(location.x, location.y, pos)]);
                    element.focus();
                }], !prohibition && ["mouseup", EventMouse.UP, function (event, mouseEvent, location, pos) {
                    selfPointer._mousePressed = false;
                    selfPointer.handleTouchesEnd([selfPointer.getTouchByXY(location.x, location.y, pos)]);
                }], !prohibition && ["mousemove", EventMouse.MOVE, function (event, mouseEvent, location, pos) {
                    selfPointer.handleTouchesMove([selfPointer.getTouchByXY(location.x, location.y, pos)]);
                    if (!selfPointer._mousePressed) {
                        mouseEvent.setButton(null);
                    }
                }], ["mousewheel", EventMouse.SCROLL, function (event, mouseEvent) {
                    mouseEvent.setScrollData(0, event.wheelDelta);
                }],
                /* firefox fix */
                ["DOMMouseScroll", EventMouse.SCROLL, function (event, mouseEvent) {
                    mouseEvent.setScrollData(0, event.detail * -120);
                }]];
                for (var i = 0; i < _mouseEventsOnElement.length; ++i) {
                    var entry = _mouseEventsOnElement[i];
                    if (entry) {
                        (function () {
                            var name = entry[0];
                            var type = entry[1];
                            var handler = entry[2];
                            element.addEventListener(name, function (event) {
                                var pos = selfPointer.getHTMLElementPosition(element);
                                var location = selfPointer.getPointByEvent(event, pos);
                                var mouseEvent = selfPointer.getMouseEvent(location, pos, type);
                                mouseEvent.setButton(event.button);

                                handler(event, mouseEvent, location, pos);

                                eventManager.dispatchEvent(mouseEvent);
                                event.stopPropagation();
                                event.preventDefault();
                            }, false);
                        })();
                    }
                }
            }

            if (window.navigator.msPointerEnabled) {
                var _pointerEventsMap = {
                    "MSPointerDown": selfPointer.handleTouchesBegin,
                    "MSPointerMove": selfPointer.handleTouchesMove,
                    "MSPointerUp": selfPointer.handleTouchesEnd,
                    "MSPointerCancel": selfPointer.handleTouchesCancel
                };

                var _loop = function _loop(eventName) {
                    var touchEvent = _pointerEventsMap[eventName];
                    element.addEventListener(eventName, function (event) {
                        var pos = selfPointer.getHTMLElementPosition(element);
                        pos.left -= document.documentElement.scrollLeft;
                        pos.top -= document.documentElement.scrollTop;

                        touchEvent.call(selfPointer, [selfPointer.getTouchByXY(event.clientX, event.clientY, pos)]);
                        event.stopPropagation();
                    }, false);
                };

                for (var eventName in _pointerEventsMap) {
                    _loop(eventName);
                }
            }

            //register touch event
            if (supportTouches) {
                var _touchEventsMap = {
                    "touchstart": function touchstart(touchesToHandle) {
                        selfPointer.handleTouchesBegin(touchesToHandle);
                    },
                    "touchmove": function touchmove(touchesToHandle) {
                        selfPointer.handleTouchesMove(touchesToHandle);
                    },
                    "touchend": function touchend(touchesToHandle) {
                        selfPointer.handleTouchesEnd(touchesToHandle);
                    },
                    "touchcancel": function touchcancel(touchesToHandle) {
                        selfPointer.handleTouchesCancel(touchesToHandle);
                    }
                };

                var registerTouchEvent = void 0;
                if (sys.browserType === sys.BROWSER_TYPE_BAIDU_GAME_SUB) {
                    _touchEventsMap = {
                        onTouchStart: _touchEventsMap.touchstart,
                        onTouchMove: _touchEventsMap.touchmove,
                        onTouchEnd: _touchEventsMap.touchend,
                        onTouchCancel: _touchEventsMap.touchcancel
                    };
                    registerTouchEvent = function registerTouchEvent(eventName) {
                        var handler = _touchEventsMap[eventName];
                        swan[eventName](function (event) {
                            if (!event.changedTouches) return;
                            var pos = selfPointer.getHTMLElementPosition(element);
                            var body = document.body;
                            pos.left -= body.scrollLeft || 0;
                            pos.top -= body.scrollTop || 0;
                            handler(selfPointer.getTouchesByEvent(event, pos));
                        });
                    };
                } else {
                    registerTouchEvent = function registerTouchEvent(eventName) {
                        var handler = _touchEventsMap[eventName];
                        element.addEventListener(eventName, function (event) {
                            if (!event.changedTouches) return;
                            var pos = selfPointer.getHTMLElementPosition(element);
                            var body = document.body;
                            pos.left -= body.scrollLeft || 0;
                            pos.top -= body.scrollTop || 0;
                            handler(selfPointer.getTouchesByEvent(event, pos));
                            event.stopPropagation();
                            event.preventDefault();
                        }, false);
                    };
                }
                for (var eventName in _touchEventsMap) {
                    registerTouchEvent(eventName);
                }
            }

            if (sys.browserType !== sys.BROWSER_TYPE_WECHAT_GAME_SUB) {
                //register keyboard event
                this._registerKeyboardEvent();
            }

            this._isRegisterEvent = true;
        },
        _registerKeyboardEvent: function _registerKeyboardEvent() {
            // baidu_game not support
        }
    });
}